@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -48,12 +48,13 @@ extern "C" {
 #define HCI_DEV_RESUME	6
 
 /* HCI device types */
-#define HCI_VHCI	0
+#define HCI_VIRTUAL	0
 #define HCI_USB		1
 #define HCI_PCCARD	2
 #define HCI_UART	3
 #define HCI_RS232	4
 #define HCI_PCI		5
+#define HCI_SDIO	6
 
 /* HCI device flags */
 enum {
@@ -97,12 +98,6 @@ enum {
 
 #define HCIINQUIRY	_IOR('H', 240, int)
 
-/* HCI timeouts */
-
-#define HCI_CONN_TIMEOUT	(HZ * 40)
-#define HCI_DISCONN_TIMEOUT	(HZ * 2)
-#define HCI_CONN_IDLE_TIMEOUT	(HZ * 60)
-
 #ifndef __NO_HCI_DEFS
 
 /* HCI Packet types */
@@ -192,6 +187,7 @@ enum {
 #define HCI_ROLE_SWITCH_PENDING			0x32
 #define HCI_SLOT_VIOLATION			0x34
 #define HCI_ROLE_SWITCH_FAILED			0x35
+#define HCI_EIR_TOO_LARGE			0x36
 
 /* ACL flags */
 #define ACL_CONT		0x01
@@ -244,6 +240,7 @@ enum {
 
 #define LMP_EDR_5SLOT	0x01
 #define LMP_SNIFF_SUBR	0x02
+#define LMP_PAUSE_ENC	0x04
 #define LMP_AFH_CAP_MST	0x08
 #define LMP_AFH_CLS_MST	0x10
 #define LMP_EDR_ESCO_2M	0x20
@@ -252,6 +249,7 @@ enum {
 
 #define LMP_EXT_INQ	0x01
 
+#define LMP_LSTO	0x01
 #define LMP_EXT_FEAT	0x80
 
 /* Link policies */
@@ -578,12 +576,11 @@ typedef struct {
 #define OCF_SNIFF_SUBRATE		0x0011
 typedef struct {
 	uint16_t	handle;
-	uint16_t	max_remote_latency;
-	uint16_t	max_local_latency;
+	uint16_t	max_latency;
 	uint16_t	min_remote_timeout;
 	uint16_t	min_local_timeout;
 } __attribute__ ((packed)) sniff_subrate_cp;
-#define SNIFF_SUBRATE_CP_SIZE 10
+#define SNIFF_SUBRATE_CP_SIZE 8
 
 /* Host Controller and Baseband */
 #define OGF_HOST_CTL		0x03
@@ -819,6 +816,8 @@ typedef struct {
 } __attribute__ ((packed)) read_transmit_power_level_rp;
 #define READ_TRANSMIT_POWER_LEVEL_RP_SIZE 4
 
+#define OCF_SET_CONTROLLER_TO_HOST_FC	0x0031
+
 #define OCF_HOST_BUFFER_SIZE		0x0033
 typedef struct {
 	uint16_t	acl_mtu;
@@ -828,18 +827,25 @@ typedef struct {
 } __attribute__ ((packed)) host_buffer_size_cp;
 #define HOST_BUFFER_SIZE_CP_SIZE 7
 
+#define OCF_HOST_NUM_COMP_PKTS		0x0035
+typedef struct {
+	uint8_t		num_hndl;
+	/* variable length part */
+} __attribute__ ((packed)) host_num_comp_pkts_cp;
+#define HOST_NUM_COMP_PKTS_CP_SIZE 1
+
 #define OCF_READ_LINK_SUPERVISION_TIMEOUT	0x0036
 typedef struct {
 	uint8_t		status;
 	uint16_t	handle;
-	uint16_t	link_sup_to;
+	uint16_t	timeout;
 } __attribute__ ((packed)) read_link_supervision_timeout_rp;
 #define READ_LINK_SUPERVISION_TIMEOUT_RP_SIZE 5
 
 #define OCF_WRITE_LINK_SUPERVISION_TIMEOUT	0x0037
 typedef struct {
 	uint16_t	handle;
-	uint16_t	link_sup_to;
+	uint16_t	timeout;
 } __attribute__ ((packed)) write_link_supervision_timeout_cp;
 #define WRITE_LINK_SUPERVISION_TIMEOUT_CP_SIZE 4
 typedef struct {
@@ -1379,8 +1385,8 @@ typedef struct {
 typedef struct {
 	uint8_t		status;
 	uint16_t	handle;
-	uint16_t	max_remote_latency;
-	uint16_t	max_local_latency;
+	uint16_t	max_tx_latency;
+	uint16_t	max_rx_latency;
 	uint16_t	min_remote_timeout;
 	uint16_t	min_local_timeout;
 } __attribute__ ((packed)) evt_sniff_subrate;
@@ -1398,6 +1404,13 @@ typedef struct {
 } __attribute__ ((packed)) extended_inquiry_info;
 #define EXTENDED_INQUIRY_INFO_SIZE 254
 
+#define EVT_LINK_SUPERVISION_TIMEOUT_CHANGE	0x38
+typedef struct {
+	uint16_t	handle;
+	uint16_t	timeout;
+} __attribute__ ((packed)) evt_link_supervision_timeout_change;
+#define EVT_LINK_SUPERVISION_TIMEOUT_SIZE 4
+
 #define EVT_TESTING			0xFE
 
 #define EVT_VENDOR			0xFF
