@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2001-2002  Nokia Corporation
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2002-2003  Stephen Crane <steve.crane@rococosoft.com>
  *
  *
@@ -69,6 +69,53 @@ static inline void sdp_list_foreach(sdp_list_t *list, sdp_list_func_t f, void *u
 		f(list->data, u);
 }
 
+/*
+ * Values of the flags parameter to sdp_record_register
+ */
+#define SDP_RECORD_PERSIST	0x01
+#define SDP_DEVICE_RECORD	0x02
+
+/*
+ * Values of the flags parameter to sdp_connect
+ */
+#define SDP_RETRY_IF_BUSY	0x01
+#define SDP_WAIT_ON_CLOSE	0x02
+#define SDP_NON_BLOCKING	0x04
+
+/*
+ * a session with an SDP server
+ */
+typedef struct {
+	int sock;
+	int state;
+	int local;
+	int flags;
+	uint16_t tid;	// Current transaction ID
+	void *priv;
+} sdp_session_t;
+
+typedef enum {
+	/*
+	 *  Attributes are specified as individual elements
+	 */
+	SDP_ATTR_REQ_INDIVIDUAL = 1,
+	/*
+	 *  Attributes are specified as a range
+	 */
+	SDP_ATTR_REQ_RANGE
+} sdp_attrreq_type_t;
+
+/*
+ * 	When the pdu_id(type) is a sdp error response, check the status value
+ * 	to figure out the error reason. For status values 0x0001-0x0006 check
+ * 	Bluetooth SPEC. If the status is 0xffff, call sdp_get_error function
+ * 	to get the real reason:
+ * 	    - wrong transaction ID(EPROTO)
+ * 	    - wrong PDU id or(EPROTO)
+ * 	    - I/O error
+ */
+typedef void sdp_callback_t(uint8_t type, uint16_t status, uint8_t *rsp, size_t size, void *udata);
+
 /*
  * create an L2CAP connection to a Bluetooth device
  * 
@@ -83,11 +130,21 @@ static inline void sdp_list_foreach(sdp_list_t *list, sdp_list_func_t f, void *u
  */
 sdp_session_t *sdp_connect(const bdaddr_t *src, const bdaddr_t *dst, uint32_t flags);
 int sdp_close(sdp_session_t *session);
+int sdp_get_socket(const sdp_session_t *session);
 
-static inline int sdp_get_socket(const sdp_session_t *s)
-{
-	return s->sock;
-}
+/*
+ * SDP transaction: functions for asynchronous search.
+ */
+sdp_session_t *sdp_create(int sk, uint32_t flags);
+int sdp_get_error(sdp_session_t *session);
+int sdp_process(sdp_session_t *session);
+int sdp_set_notify(sdp_session_t *session, sdp_callback_t *func, void *udata);
+
+int sdp_service_search_async(sdp_session_t *session, const sdp_list_t *search, uint16_t max_rec_num);
+int sdp_service_attr_async(sdp_session_t *session, uint32_t handle, sdp_attrreq_type_t reqtype, const sdp_list_t *attrid_list);
+int sdp_service_search_attr_async(sdp_session_t *session, const sdp_list_t *search, sdp_attrreq_type_t reqtype, const sdp_list_t *attrid_list);
+
+uint16_t sdp_gen_tid(sdp_session_t *session);
 
 /*
  * find all devices in the piconet
@@ -267,7 +324,7 @@ void sdp_set_url_attr(sdp_record_t *rec, const char *clientExecURL, const char *
  * 
  *  INPUT :
  * 
- *    sdp_list_t *search_list
+ *    sdp_list_t *search
  *      list containing elements of the search
  *      pattern. Each entry in the list is a UUID
  *      of the service to be searched
@@ -290,7 +347,7 @@ void sdp_set_url_attr(sdp_record_t *rec, const char *clientExecURL, const char *
  *      non-zero service handles. It is a singly linked list of
  *      service record handles (uint16_t)
  */
-int sdp_service_search_req(sdp_session_t *session, const sdp_list_t *search_list, uint16_t max_rec_num, sdp_list_t **rsp_list);
+int sdp_service_search_req(sdp_session_t *session, const sdp_list_t *search, uint16_t max_rec_num, sdp_list_t **rsp_list);
 
 /*
  *  a service attribute request. 
@@ -325,18 +382,6 @@ int sdp_service_search_req(sdp_session_t *session, const sdp_list_t *search_list
  *      -1
  *        The request completed unsuccessfully due to a timeout
  */
-
-typedef enum {
-	/*
-	 *  Attributes are specified as individual elements
-	 */
-	SDP_ATTR_REQ_INDIVIDUAL = 1,
-	/*
-	 *  Attributes are specified as a range
-	 */
-	SDP_ATTR_REQ_RANGE
-} sdp_attrreq_type_t;
-
 sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle, sdp_attrreq_type_t reqtype, const sdp_list_t *attrid_list);
 
 /*
@@ -346,7 +391,7 @@ sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle, sdp_
  * 
  *  INPUT :
  * 
- *    sdp_list_t *search_list
+ *    sdp_list_t *search
  *      Singly linked list containing elements of the search
  *      pattern. Each entry in the list is a UUID(DataTypeSDP_UUID16)
  *      of the service to be searched
@@ -380,7 +425,7 @@ sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle, sdp_
  *      service(s) found. Each element of this list is of type
  *      sdp_record_t *.
  */
-int sdp_service_search_attr_req(sdp_session_t *session, const sdp_list_t *search_list, sdp_attrreq_type_t reqtype, const sdp_list_t *attrid_list, sdp_list_t **rsp_list);
+int sdp_service_search_attr_req(sdp_session_t *session, const sdp_list_t *search, sdp_attrreq_type_t reqtype, const sdp_list_t *attrid_list, sdp_list_t **rsp_list);
 
 /*
  * Allocate/free a service record and its attributes
@@ -535,14 +580,6 @@ static inline int sdp_get_icon_url(const sdp_record_t *rec, char *str, int len)
 	return sdp_get_string_attr(rec, SDP_ATTR_ICON_URL, str, len);
 }
 
-/*
- * Generate unique transaction identifiers
- */
-static inline uint16_t sdp_gen_tid(sdp_session_t *session)
-{
-	return session->tid++;
-}
-
 sdp_record_t *sdp_extract_pdu(const uint8_t *pdata, int *scanned);
 sdp_data_t *sdp_extract_string(uint8_t *, int *);
 
@@ -568,11 +605,6 @@ void sdp_pattern_add_uuidseq(sdp_record_t *rec, sdp_list_t *seq);
 
 int sdp_send_req_w4_rsp(sdp_session_t *session, uint8_t *req, uint8_t *rsp, uint32_t reqsize, uint32_t *rspsize);
 
-typedef struct {
-	uint8_t length;
-	unsigned char data[16];
-} __attribute__ ((packed)) sdp_cstate_t;
-
 #ifdef __cplusplus
 }
 #endif
