@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -376,6 +376,14 @@ char *bt_compidtostr(int compid)
 		return "Bluetooth SIG, Inc";
 	case 64:
 		return "Seiko Epson Corporation";
+	case 65:
+		return "Integrated Silicon Solution Taiwain, Inc.";
+	case 66:
+		return "CONWISE Technology Corporation Ltd";
+	case 67:
+		return "PARROT SA";
+	case 68:
+		return "Socket Communications";
 	case 65535:
 		return "internal use";
 	default:
