@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2006  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -36,6 +36,7 @@
 #include <string.h>
 
 #include <sys/param.h>
+#include <sys/uio.h>
 #include <sys/poll.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -93,7 +94,7 @@ static int hci_str2bit(hci_map *map, char *str, unsigned int *val)
 
 static char *hci_uint2str(hci_map *m, unsigned int val) 
 {
-	char *str = bt_malloc(50);
+	char *str = malloc(50);
 	char *ptr = str;
 
 	if (!str)
@@ -137,8 +138,8 @@ static int hci_str2uint(hci_map *map, char *str, unsigned int *val)
 char *hci_dtypetostr(int type)
 {
 	switch (type) {
-	case HCI_VHCI:
-		return "VHCI";
+	case HCI_VIRTUAL:
+		return "VIRTUAL";
 	case HCI_USB:
 		return "USB";
 	case HCI_PCCARD:
@@ -149,8 +150,10 @@ char *hci_dtypetostr(int type)
 		return "RS232";
 	case HCI_PCI:
 		return "PCI";
+	case HCI_SDIO:
+		return "SDIO";
 	default:
-		return "UKNW";
+		return "UNKNOWN";
 	}
 }
 
@@ -171,7 +174,7 @@ static hci_map dev_flags_map[] = {
 
 char *hci_dflagstostr(uint32_t flags)
 {
-	char *str = malloc(50);
+	char *str = bt_malloc(50);
 	char *ptr = str;
 	hci_map *m = dev_flags_map;
 
@@ -280,7 +283,7 @@ static hci_map link_mode_map[] = {
 
 char *hci_lmtostr(unsigned int lm)
 {
-	char *s, *str = malloc(50);
+	char *s, *str = bt_malloc(50);
 	if (!str)
 		return NULL;
 
@@ -290,7 +293,7 @@ char *hci_lmtostr(unsigned int lm)
 
 	s = hci_bit2str(link_mode_map, lm);
 	if (!s) {
-		free(str);
+		bt_free(str);
 		return NULL;
 	}
 
@@ -582,7 +585,7 @@ static hci_map lmp_features_map[8][9] = {
 	{	/* Byte 5 */
 		{ "<5-slot EDR ACL>",	LMP_EDR_5SLOT	},	/* Bit 0 */
 		{ "<sniff subrating>",	LMP_SNIFF_SUBR	},	/* Bit 1 */
-		{ "<no. 42>",		0x04		},	/* Bit 2 */
+		{ "<pause encryption>",	LMP_PAUSE_ENC	},	/* Bit 2 */
 		{ "<AFH cap. master>",	LMP_AFH_CAP_MST	},	/* Bit 3 */
 		{ "<AFH class. master>",LMP_AFH_CLS_MST	},	/* Bit 4 */
 		{ "<EDR eSCO 2 Mbps>",	LMP_EDR_ESCO_2M	},	/* Bit 5 */
@@ -592,9 +595,23 @@ static hci_map lmp_features_map[8][9] = {
 	},
 	{	/* Byte 6 */
 		{ "<extended inquiry>",	LMP_EXT_INQ	},	/* Bit 0 */
+		{ "<no. 49>",		0x02		},	/* Bit 1 */
+		{ "<no. 50>",		0x04		},	/* Bit 2 */
+		{ "<no. 51>",		0x08		},	/* Bit 3 */
+		{ "<no. 52>",		0x10		},	/* Bit 4 */
+		{ "<no. 53>",		0x20		},	/* Bit 5 */
+		{ "<no. 54>",		0x40		},	/* Bit 6 */
+		{ "<no. 55>",		0x80		},	/* Bit 7 */
 		{ NULL }
 	},
 	{	/* Byte 7 */
+		{ "<LSTO>",		LMP_LSTO	},	/* Bit 1 */
+		{ "<no. 57>",		0x02		},	/* Bit 1 */
+		{ "<no. 58>",		0x04		},	/* Bit 2 */
+		{ "<no. 59>",		0x08		},	/* Bit 3 */
+		{ "<no. 60>",		0x10		},	/* Bit 4 */
+		{ "<no. 61>",		0x20		},	/* Bit 5 */
+		{ "<no. 62>",		0x40		},	/* Bit 6 */
 		{ "<extended features>",LMP_EXT_FEAT	},	/* Bit 7 */
 		{ NULL }
 	},
@@ -663,6 +680,8 @@ int hci_for_each_dev(int flag, int (*func)(int dd, int dev_id, long arg), long a
 		goto done;
 	}
 
+	memset(dl, 0, HCI_MAX_DEV * sizeof(*dr) + sizeof(*dl));
+
 	dl->dev_num = HCI_MAX_DEV;
 	dr = dl->dev_req;
 
@@ -747,6 +766,8 @@ int hci_devinfo(int dev_id, struct hci_dev_info *di)
 	if (dd < 0)
 		return dd;
 
+	memset(di, 0, sizeof(struct hci_dev_info));
+
 	di->dev_id = dev_id;
 	ret = ioctl(dd, HCIGETDEVINFO, (void *) di);
 
@@ -761,6 +782,8 @@ int hci_devba(int dev_id, bdaddr_t *bdaddr)
 {
 	struct hci_dev_info di;
 
+	memset(&di, 0, sizeof(di));
+
 	if (hci_devinfo(dev_id, &di))
 		return -1;
 
@@ -855,6 +878,7 @@ int hci_open_dev(int dev_id)
 		return dd;
 
 	/* Bind socket to the HCI device */
+	memset(&a, 0, sizeof(a));
 	a.hci_family = AF_BLUETOOTH;
 	a.hci_dev = dev_id;
 	if (bind(dd, (struct sockaddr *) &a, sizeof(a)) < 0)
@@ -976,13 +1000,13 @@ int hci_send_req(int dd, struct hci_request *r, int to)
 			if (cs->opcode != opcode)
 				continue;
 
-			if (cs->status) {
-				errno = EIO;
-				goto failed;
-			}
-
-			if (r->event != EVT_CMD_STATUS)
+			if (r->event != EVT_CMD_STATUS) {
+				if (cs->status) {
+					errno = EIO;
+					goto failed;
+				}
 				break;
+			}
 
 			r->rlen = MIN(len, r->rlen);
 			memcpy(r->rparam, ptr, r->rlen);
@@ -2038,7 +2062,7 @@ int hci_read_link_supervision_timeout(int dd, uint16_t handle, uint16_t *timeout
 		return -1;
 	}
 
-	*timeout = rp.link_sup_to;
+	*timeout = rp.timeout;
 	return 0;
 }
 
@@ -2049,8 +2073,8 @@ int hci_write_link_supervision_timeout(int dd, uint16_t handle, uint16_t timeout
 	struct hci_request rq;
 
 	memset(&cp, 0, sizeof(cp));
-	cp.handle      = handle;
-	cp.link_sup_to = timeout;
+	cp.handle  = handle;
+	cp.timeout = timeout;
 
 	memset(&rq, 0, sizeof(rq));
 	rq.ogf    = OGF_HOST_CTL;
